/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file AnnotationFinalizer.h
 *  \author <a href="mailto:moritz.becker@pallas-ludens.com">
 *      Moritz Becker</a>
 *  \date 22.07.2014
 *  Declaraion of parameter class AnnotationFinalizer.
 */
#ifndef _ANNOTATIONFINALIZER_H_
#define _ANNOTATIONFINALIZER_H_

#ifdef _MSC_VER
#ifdef annotationfinalizer_EXPORTS
/// Visual C++ specific code
#define annotationfinalizer_DECLDIR __declspec(dllexport)
#else
#define annotationfinalizer_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define annotationfinalizer_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

#include <charon-utils/Annotation.hxx>

/// Postprocessing for annotation
/** Rules to decide which annotation can be flagged as final due to enough
 *  matches etc.
 */
class annotationfinalizer_DECLDIR AnnotationFinalizer : public ParameteredObject {
public:
	/// default constructor
	/// \param name             instance name
	AnnotationFinalizer(const std::string& name = "");

	/// Annotations
	InputSlot< annotation::AnnotationList > annotations;

	/// vector of image file names 
	InputSlot< std::vector< std::string > > filepaths;

	/// Final annotations
	OutputSlot< annotation::AnnotationList > annotationsFinal;

	/// Threshold for match distance
	Parameter< float > minMatchDistance;

	/// Threshold for the forth coordinate match distance
	Parameter< float > minMatchDistanceOutlier;

	/// Threshold for the forth coordinate match distance
	Parameter< float > maxMatchDistance;

	/// Threshold for match distance (relative)
	Parameter< float > relativeMatchDistance;

	/// Threshold for match count
	Parameter< int > thresholdMatchCount;

	/// Maximal size of boxes given in percent
	Parameter< float > maxSize;

	/// Propagate similar annotations
	Parameter< bool > propagateNeighbours;

	/// Also save deleted annotations (marked as deleted)
	Parameter< bool > saveDeleted;



protected:
	/// Update object.
	virtual void execute();

	void getMedians(std::vector<std::pair<float, float>> &medianPoints, const std::vector< const annotation::AnnotationPoints* > &matchList);
	
	bool outlierMatchCount(
				const annotation::Frame* frame,
				const annotation::AnnotationPoints* box, 
				std::vector<std::pair<float, float>> &medianPoints,
				int width, int height);
};

#endif /* _ANNOTATIONFINALIZER_H_ */
