/*  Copyright (C) 2014 Moritz Becker

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file DrawBoxes.h
 *  Declaration of the parameter class DrawBoxes.
 *  \author <a href="mailto:moritz.becker@iwr.uni-heidelberg.de">
 *      Moritz Becker</a>
 *  \date 19.05.2014
 */

#ifndef _DRAWANNOTATIONS_H_
#define _DRAWANNOTATIONS_H_

#ifdef _MSC_VER
#ifdef drawannotations_EXPORTS
/// Visual C++ specific code
#define drawannotations_DECLDIR __declspec(dllexport)
#else
#define drawannotations_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define drawannotations_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

#include <charon-utils/Annotation.hxx>

/// Draw boxes on given images
/** Draw boxes on given images
 */
template <typename T>
class drawannotations_DECLDIR DrawAnnotations :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	DrawAnnotations(const std::string& name = "");
	
	/// annotation values
	InputSlot< annotation::AnnotationList > annotations;

	/// vector of image file names 
	InputSlot< std::vector< std::string > > filepaths;

	/// Output images
	OutputSlot< cimg_library::CImgList<T> > images;

	/// draws a closed loop of list of points
	Parameter< bool > closeLoop;

	/// JSON data contains boxes
	Parameter< bool > isBox;

	/// draw final results
	Parameter< bool > resultsFinal;
	/// draw add results
	Parameter< bool > resultsAdd;
	/// draw failed results
	Parameter< bool > resultsFailed;
	/// draw edit results
	Parameter< bool > resultsEdit;
	/// draw done results
	Parameter< bool > resultsDone;
	/// draw deleted results
	Parameter< bool > resultsDeleted;

	/// Save all input images whether with or without drawn boxes. Otherwise,
	/// only images with boxes are saved.
	Parameter< bool > processAll;

	/// Draw without states
	Parameter< bool > processWithoutStates;



protected:
	/// Update object.
	virtual void execute();
};

#endif // _DRAWANNOTATIONS_H_
